import {
  Directive,
  OnInit,
  OnDestroy,
  Self,
  Inject,
  ChangeDetectorRef
} from '@angular/core';
import { Subject, fromEvent } from 'rxjs';
import { CdkVirtualScrollViewport } from '@angular/cdk/scrolling';
import { debounceTime, takeUntil } from 'rxjs/operators';
import { DOCUMENT } from '@angular/common';

@Directive({
  // tslint:disable-next-line: directive-selector
  selector: 'cdk-virtual-scroll-viewport'
})
export class CdkVirtualScrollViewportPatchDirective
  implements OnInit, OnDestroy {
  protected readonly destroySubject = new Subject();

  constructor(
    @Self()
    @Inject(CdkVirtualScrollViewport)
    private readonly viewportComponent: CdkVirtualScrollViewport,
    @Inject(DOCUMENT)
    private readonly document: Document
  ) {}

  ngOnInit() {
    fromEvent(this.document.defaultView, 'resize')
      .pipe(debounceTime(10), takeUntil(this.destroySubject))
      .subscribe(() => {
        this.viewportComponent.checkViewportSize();
      });
  }

  ngOnDestroy() {
    this.destroySubject.next();
    this.destroySubject.complete();
  }
}
